<?php

/**
 * @file
 * Contains \Drupal\domino\TestUsersSubscriber.
 */

namespace Drupal\domino;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\State\StateInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Ensures actual status of test users in the system.
 */
class TestUsersSubscriber implements EventSubscriberInterface {

  /**
   * Domino config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * State handler instance.
   *
   * @var \Drupal\Core\State\StateInterface;
   */
  protected StateInterface $state;

  /**
   * Time object instance.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected TimeInterface $time;

  /**
   * Test users handler.
   *
   * @var \Drupal\domino\TestUsers
   */
  protected TestUsers $testUsers;

  /**
   * Constructs a new TestUsersSubscriber instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory instance.
   * @param \Drupal\Core\State\StateInterface $state
   *   State handler instance.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   Time object instance.
   * @param \Drupal\domino\TestUsers $test_users
   *   Test users object instance.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, TimeInterface $time, TestUsers $test_users) {
    $this->config = $config_factory->get('domino.settings');
    $this->state = $state;
    $this->time = $time;
    $this->testUsers = $test_users;
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['ensureTestUsersStatus'];
    return $events;
  }

  /**
   * Ensure actual status of test users in the system.
   *
   * Makes sure that test users exist, contain actual
   * password and are activated or deactivated.
   */
  public function ensureTestUsersStatus() {
    // Regularly ensure that test users are in the actual activation status
    // and their password is correct.
    $last_check_timestamp = $this->state->get('domino.last_test_users_check', 0);
    $current_timestamp = $this->time->getRequestTime();
    // Defaults to 5 minutes.
    $check_frequency = $this->config->get('test_users_check_frequency');
    if ($last_check_timestamp + $check_frequency < $current_timestamp) {
      $this->testUsers->ensureTestUsersPassword();
      $this->testUsers->ensureTestUsersActivationStatus();
      $this->state->set('domino.last_test_users_check', $current_timestamp);
    }
  }

}
